! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! In this example add dataset is created (opened) containing
! two 3-dimensional arrays.
!
! It illustrates the use of the derived types DataSetT and ArrayT.
!
! The second array has the same data type as the first; this
! is ensured by using the arrayDataType() function to determine
! the data type of the first array.
!
! The first array is filled with unique data before the
! dataset is released (closed).
program example_addarray

  use dal
  use errorhandling

  implicit none

  type(DataSetT) set
  type(ArrayT) arr1, arr2
  integer(kind=int32), dimension(:,:,:), pointer :: a1, a2
  integer, dimension(3), parameter :: s = (/ 3,4,2 /)
  integer :: i,j,k,n

  ! create a set
  set = dataSet("test.dat",CREATE)
  arr1 = addArray(set, "array1", INTEGER32, dimensions=s )
  arr2 = addArray(set, "array2", arrayDataType( arr1 ), dimensions=s )

  ! fill with unique numbers
  a1 => int32Array3Data(arr1)
  a2 => int32Array3Data(arr1)

  n = 0
  do k=0,1
    do j=0,3
      do i=0,2 
        a1(i,j,k) = n
        a2(i,j,k) = a1(i,j,k) + 1
        n = n + 1
      end do
    end do
  end do

  call release(arr1)
  call release(arr2)
  call release(set)

end program example_addarray
