// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
/*************************************************************** 
* 
* DalArrayData.cc           & XMM Science Analysis System (SAS) 
* 
* Class representing DAL Array Data.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: DalArrayData.cc,v 1.2 2004/03/05 14:43:00 eojero Exp $
* 
*****************************************************************/ 

#include <Tracer.h>
#include <vectorutil.h>
#include <errstr.h>


#include "DalArrayData.h"


DalArrayData::DalArrayData(
	const Array *			parent,
	const longVec&			from,
	const longVec&			count )
	:
		DalSeekable<longVec>( from, count ),
		DalChild<Array>( parent )
{
	PROC
	ENDPROC
}


static bool
singular( const longVec& pos )
{
	PROC
	for( unsigned int i = 0; i < pos.size(); ++i )
	{
		if( pos[ i ] )
		{
			ALTRETURNV( false );
		} // if
	} // for
	RETURN true;
}


unsigned long
DalArrayData::position( const longVec& pos ) const
{
	PROC

	unsigned dim = this -> DalChild<Array>::parent()->size().size();

	if (pos.size() != dim)
		errstr << "the position into array " << this -> DalChild<Array>::parent()->name() << " has dimension " << pos.size() << " whereas the data has dimension " << dim << Sas::error("invalidPosition");

	// TEMP SOLUTION TO SPR-1166
	if( singular( pos ) )
	{
		ALTRETURNV( 0 );
	} // if

	unsigned long idx = 0, mult = 1;

	const longVec& fromVec = from(), countVec = count();

	for (unsigned i = dim, ix = dim-1; i; --i,--ix) {

		unsigned long from = ( fromVec == zero ? 0 : fromVec[ix] );
		unsigned long count = ( countVec == zero ? size()[ix] - from : countVec[ix] );
		
		SAS_DEBUG( clog << pos[ix] << ' ' << from << ' ' << count << endl; )

		if (pos[ix] < from || pos[ix] >= from + count)
			errstr << "cannot access position " << pos <<" since the array data starts at " << fromVec << " and has length " << countVec << Sas::error("invalidPosition");

		unsigned long dist = pos[ix] - from;

		idx += dist*mult;

		if (ix) mult *= count;
	}

	RETURN idx;
}



