// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*************************************************************** 
* 
* DalAttribute.h             & XMM Science Analysis System (SAS) 
* 
* Class representing an Attribute. 
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA, and Marco Beijersbergen, CRI
* 
* $Id: DalAttribute.h,v 1.2 2004/03/05 14:43:00 eojero Exp $
* 
*****************************************************************/ 
#ifndef DAL_ATTRIBUTE_H
#define DAL_ATTRIBUTE_H


#include <stringutil.h>

#include "machine_specific_types.h"
#include "Dal.h"
#include "DalChild.h"
#include "DalLabelled.h"
#include "DalDal.h"
#include "DalAttributable.h"


class Value;


class DalAttribute : public virtual Attribute, public virtual DalLabelled, public DalDal, public DalChild<Attributable>
{
	public :

		// Constructors
		DalAttribute(
			const Attribute*	org,
			const DalAttributable*	parent );

		DalAttribute(
			const std::string &		name,
			const std::string &		units,
			const std::string &		comment,
			const DalAttributable *	parent );

		virtual ~DalAttribute();

		virtual Attribute & operator=( int		value );
		virtual Attribute & operator=( double		value );
		virtual Attribute & operator=( bool 		value );
		virtual Attribute & operator=( const char *	value );
		virtual Attribute & operator=( const std::string &	value );
		virtual Attribute & operator=( const Attribute& );

		int asInt() const;
		double asReal() const;
		std::string asString() const;
		bool asBool() const;

		const std::string & units() const;
		void units(const std::string& u);

		DataType dataType() const;

		void dataType(DataType type);

		char* addressOfValue() const;

		unsigned int dataSize() const;

		unsigned int attributeNumber() const;

		void printOn( ostream & os ) const;

		std::string parseName( const std::string& compoundName ) const;

		std::string qualifiedName() const;

		void release();

	private :

		Value*		_value;
		std::string		_units;

		void checkValue() const;
};

#endif















