// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*************************************************************** 
* 
* HiLowCellData.h            & XMM Science Analysis System (SAS) 
* 
* Class representing the Data in a Column Cell. 
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: HiLowCellData.h,v 1.2 2004/03/05 14:43:02 eojero Exp $ 
* 
*****************************************************************/ 
#ifndef HILOW_CELL_DATA
#define HILOW_CELL_DATA


#include <Dal.h>

#include "HiLowData.h"
#include "HiLowDataStorage.h"
#include "DalCellData.h"
#include "DalColumn.h"


class HiLowCellData : public DalCellData, public HiLowData<DalColumn>
{
	public :

		HiLowCellData(
			HiLowDataStorage *		data,
			const DalColumn *		parent,
			unsigned long			rowNumber,
			unsigned long 			from,
			unsigned long 			count,
			AccessMode			accessMode );

		virtual ~HiLowCellData();

		int8 * int8Data() const;
		uint16 * uint16Data() const;
		int16 * int16Data() const;
		uint32 * uint32Data() const;
		int32 * int32Data() const;
		real32 * real32Data() const;
		real64 * real64Data() const;
		bool8 * boolData() const;
		char * stringData() const;

		void size( unsigned long size );
		unsigned long size() const;
		unsigned long row() const;
		unsigned long elements() const;
		void seek( unsigned long from, unsigned long count );

		void dumpData( char ** values, unsigned long from, unsigned long count ) const;
		void dumpData( bool8 * values, unsigned long from, unsigned long count ) const;
		void dumpData( int8 * values, unsigned long from, unsigned long count ) const;
		void dumpData( uint16 * values, unsigned long from, unsigned long count ) const;
		void dumpData( int16 * values, unsigned long from, unsigned long count ) const;
		void dumpData( uint32 * values, unsigned long from, unsigned long count ) const;
		void dumpData( int32 * values, unsigned long from, unsigned long count ) const;
		void dumpData( real32 * values, unsigned long from, unsigned long count ) const;
		void dumpData( real64 * values, unsigned long from, unsigned long count ) const;
		void loadData( char ** values, unsigned long from, unsigned long count ) const;
		void loadData( bool8 * values, unsigned long from, unsigned long count ) const;
		void loadData( int8 * values, unsigned long from, unsigned long count ) const;
		void loadData( uint16 * values, unsigned long from, unsigned long count ) const;
		void loadData( int16 * values, unsigned long from, unsigned long count ) const;
		void loadData( uint32 * values, unsigned long from, unsigned long count ) const;
		void loadData( int32 * values, unsigned long from, unsigned long count ) const;
		void loadData( real32 * values, unsigned long from, unsigned long count ) const;
		void loadData( real64 * values, unsigned long from, unsigned long count ) const;
		void copyData( unsigned long from, unsigned long to, unsigned long count ) const;

};


#endif


