// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*************************************************************** 
* 
* HiLowDataStorage.h              & XMM Science Analysis System (SAS) 
* 
* Class representing data storage object.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: HiLowDataStorage.h,v 1.2 2004/03/05 14:43:02 eojero Exp $ 
* 
*****************************************************************/ 
#ifndef HILOW_DATA_STORAGE_H
#define HILOW_DATA_STORAGE_H


#include "Dal.h"
#include "DataManipulator.h"
#include "DalDal.h"
#include "DalSeekable.h"

class HiLowValues;


class HiLowDataStorage : public DalDal, public virtual DataManipulator
{
public :

  // Constructors
  HiLowDataStorage(
     const std::string&   name,
     DataType   dataType,
     unsigned long   size );

  virtual ~HiLowDataStorage();

  int8 * int8Data( unsigned long from, unsigned long count );
  uint16 * uint16Data( unsigned long from, unsigned long count );
  int16 * int16Data( unsigned long from, unsigned long count );
  uint32 * uint32Data( unsigned long from, unsigned long count );
  int32 * int32Data( unsigned long from, unsigned long count );
  real32 * real32Data( unsigned long from, unsigned long count );
  real64 * real64Data( unsigned long from, unsigned long count );
  bool8 * boolData( unsigned long from, unsigned long count );
  char * stringData( unsigned long from, unsigned long count );
  unsigned int dataSize() const;
  DataType dataType() const;
  void size( unsigned long size );
  unsigned long size() const;
  unsigned long from() const;
  unsigned long count() const;
  void insertBytes( unsigned long pos, unsigned long count=1 );
  void deleteBytes( unsigned long pos, unsigned long count=1 );
  void copyBytes( unsigned long from, unsigned long to, unsigned long count );
  void printOn( ostream & os, const std::string& sep="" );
  void release( unsigned long from, unsigned long count );
  void release();
  //void initialise( unsigned long from, unsigned long count );
  void initialise(
    const DalSeekable<unsigned long>& oldSeek,
    const DalSeekable<unsigned long>& seek );

private :

  std::string   _name;
  HiLowValues *  _values;
  void cannotConvertTo(const std::string& toType) const;

};



#endif

