// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
/*************************************************************** 
* 
* HiLowTable.cc             & XMM Science Analysis System (SAS) 
* 
* Class representing DAL Dataset Table.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: HiLowTable.cc,v 1.2 2004/03/05 14:43:02 eojero Exp $
* 
*****************************************************************/ 


#include <errstr.h>
#include <Tracer.h>
#include <stringutil.h>


#include "HiLowTable.h"
#include "HiLowColumn.h"
#include "DalDataSet.h"


HiLowTable::HiLowTable(
			const string &	name,
			unsigned long	rows,
			const string &	label,
			DalDataSet *	parent )

		:
		DalTable( name, label, rows, parent )
{
	PROC
	ENDPROC

}


HiLowTable::HiLowTable(
			const Table& table,
			DalDataSet* parent,
			string newName)
		:
		DalTable( newName.empty() ? table.name() : newName, table.label(), table.rows(), parent )
{
	PROC

	operator=(table);

	ENDPROC
}


HiLowTable::~HiLowTable()
{
	PROC

	// *******************************
	// Release and delete the columns. 
	// *******************************
	release();

	for(
		ColumnList::iterator i = _columns.begin();
		i != _columns.end();
		++i )
	{
		delete (*i);
	} // for

	ENDPROC

}


DalTable &
HiLowTable::operator=( const Table & table )
{
	PROC

	(Attributable&)*this = table;

	for (ColumnList::iterator i = _columns.begin(); i != _columns.end(); ++i)
	{
		delete (*i);
	} // for

	this -> _rows = table.rows();

	for (unsigned i = 0; i < table.columns(); ++i)
	{
		const Column *org = table.column(i);
		add( org, org -> name() );
		//_columns.push_back( new HiLowColumn( *org, this ));
	} // for

	RETURN *this;

}


DalColumn *
HiLowTable::makeColumn(
		const string &			name,
		Data::DataType			dataType,
		const string &			label,
		const string &			units,
		const longVec &			size,
		DalTable *			parent )
{
	PROC
	RETURN new HiLowColumn(
				name,
				dataType,
				label,
				units,
				size,
				parent );

}


void
HiLowTable::forEachSubTable( void(*callThisFunction)( Table * table ))
{
	PROC

	const char * dal_rows = getenv( "SAS_ROWS" );
	const unsigned long seekSize = (dal_rows ? atoi( dal_rows ) : 1 );

	//cout << seekSize << endl;

	for( unsigned long from = 0; from < _rows; from = from + seekSize )
	{
		const unsigned long seekThisMany = (( from + seekSize <= _rows) ? seekSize : _rows - from );
		//cout << "seek(" << from << "," << seekThisMany << ")"  << endl;

		const unsigned int numberOfColumns = _columns.size();

		for(
			unsigned int col = 0;
			col < numberOfColumns;
			++col )
		{
			_columns[ col ] -> seek( from, seekThisMany );
		} // for
		callThisFunction( this );
	} // for
	ENDPROC
}

/*
void
HiLowTable::seek(
	unsigned long		from,
	unsigned long		count )
{
	PROC
	for( unsigned int i = 0; i < _columns.size(); ++i )
	{
		_columns[ i ] -> seek( from, count );
	} // for
	DalSeekable<unsigned long>::seek( from, count );
	ENDPROC
}
*/

//void
//HiLowTable::releaseAll()
//{
//	PROC
//	for(
//		ColumnList::iterator i = _columns.begin();
//		i != _columns.end();
//		++i )
//	{
//		(*i) -> releaseAll();
//	} // for
//	ENDPROC
//}


