// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
#include<cassert>
#include <Dal.h>

int main()
{
  DataSet* set = dataSetServer->open("test.dat",DataSet::Create);

  *set->addAttribute("sbool","set bool comment") = false;
  *set->addAttribute("sint","set int comment","int unit") = 1;
  *set->addAttribute("sreal","set real comment","real unit") = 2.5;
  *set->addAttribute("sstring","set string comment") = "a set string";

  Table* table1 = set->addTable("table1",10);
  table1->addAttribute(set->attribute("sbool"));

  unsigned long nRows = 10;
  Table* table2 = set->addTable("table2",nRows);
  table2->addAttributes(set);
  Column *col = table2->addColumn("int32",Column::Int32,"int32 data","dm");
  ColumnData *d = col->data();
  int32 *i32 = d->int32Data();
  for (unsigned long i = 0; i < nRows; ++i)
    i32[i] = 3*i;
  delete d;

  set->add(table2,"table3");

  dataSetServer->close(set);

  dataSetServer->copy("test.dat","test1.dat");

  set = dataSetServer->open("test1.dat",DataSet::Read);
  Table *table3 = set->table("table3");
  assert( table3->attribute("sbool")->asBool() == false );
  assert( table3->attribute("sint")->asInt() == 1 );
  assert( table3->attribute("sreal")->asReal() == 2.5 );
  assert( table3->attribute("sstring")->asString() == "a set string" );

  col = table3->column("int32");
  assert( col->units() == "dm" );
  assert( col->rows() == nRows );
  assert( col->label() == "int32 data" );
  d = col->data();
  assert( d->count()  == nRows );
  i32 = d->int32Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( i32[i] == 3*int(i) );
  delete d;

  dataSetServer->close(set);

  return 0;
}
