// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
#include <Dal.h>
#include<cassert>
/* AI gcc4 porting
 *  strncmp was not declared in this scope
 * include string.h
 */
#include <string.h>

int main()
{
  DataSet* set = dataSetServer->open("test.dat",DataSet::Create);
  Table* tab = set->addTable("events",100);

  Column *col = tab->addColumn("bool",Column::Bool);
  ColumnData *data = col->data();
  bool8* b8 = data->boolData();
  for (unsigned i = 0; i < col->rows(); ++i)
    b8[i] = (i%2)?true:false;
  delete data;

  col = tab->addColumn("int8",Column::Int8);
  data = col->data();
  int8* i8 = data->int8Data();
  for (unsigned i = 0; i < col->rows(); ++i)
    i8[i] = i;
  delete data;

  col = tab->addColumn("int16",Column::Int16);
  data = col->data();
  int16* i16 = data->int16Data();
  for (unsigned i = 0; i < col->rows(); ++i)
    i16[i] = i;
  delete data;

  col = tab->addColumn("int32",Column::Int32);
  data = col->data();
  int32* i32 = data->int32Data();
  for (unsigned i = 0; i < col->rows(); ++i)
    i32[i] = i;
  delete data;

  col = tab->addColumn("real32",Column::Real32);
  data = col->data();
  real32* r32 = data->real32Data();
  for (unsigned i = 0; i < col->rows(); ++i)
    r32[i] = i;
  delete data;

  col = tab->addColumn("real64",Column::Real64);
  data = col->data();
  real64* r64 = data->real64Data();
  for (unsigned i = 0; i < col->rows(); ++i)
    r64[i] = i;
  delete data;

  vector<unsigned long> strlen;
  strlen.push_back(10);
  col = tab->addColumn("string",Column::DString,"","",strlen);
  data = col->data();
  char* s = data->stringData();
  for (unsigned i = 0; i < col->rows() * col -> elements(); ++i)
     s[ i ] = 'a' + (i % 26 );
  delete data;

  // Check values before copying
  data = tab -> column( "bool" ) -> data();
  b8 = data->boolData();
  assert(b8[0] == false);
  assert(b8[1] == true);
  assert(b8[2] == false);
  assert(b8[3] == true);
  assert(b8[4] == false);
  delete data;

  data = tab -> column( "int8" ) -> data();
  i8 = data->int8Data();
  assert(i8[0] == 0);
  assert(i8[1] == 1);
  assert(i8[2] == 2);
  assert(i8[3] == 3);
  assert(i8[4] == 4);
  delete data;

  data = tab -> column( "int16" ) -> data();
  i16 = data->int16Data();
  assert(i16[0] == 0);
  assert(i16[1] == 1);
  assert(i16[2] == 2);
  assert(i16[3] == 3);
  assert(i16[4] == 4);
  delete data;

  data = tab -> column( "int32" ) -> data();
  i32 = data->int32Data();
  assert(i32[0] == 0);
  assert(i32[1] == 1);
  assert(i32[2] == 2);
  assert(i32[3] == 3);
  assert(i32[4] == 4);
  delete data;

  data = tab -> column( "real32" ) -> data();
  r32 = data->real32Data();
  assert(r32[0] == 0);
  assert(r32[1] == 1);
  assert(r32[2] == 2);
  assert(r32[3] == 3);
  assert(r32[4] == 4);
  delete data;

  data = tab -> column( "real64" ) -> data();
  r64 = data->real64Data();
  assert(r64[0] == 0);
  assert(r64[1] == 1);
  assert(r64[2] == 2);
  assert(r64[3] == 3);
  assert(r64[4] == 4);
  delete data;

/* AI gcc porting
 *  deprecated conversion from string constant to char*
 * cast to (char*)
 */
  data = tab -> column( "string" ) -> data();
  s = data->stringData();
  char * s0 = (char*) "abcdefghij";
  char * s1 = (char*) "klmnopqrst";
  char * s2 = (char*) "uvwxyzabcd";
  char * s3 = (char*) "efghijklmn";
  char * s4 = (char*) "opqrstuvwx";
  //char * s5 = (char*) "yzabcdefgh";
  char * s6 = (char*) "ijklmnopqr";
  char * s7 = (char*) "stuvwxyzab";
  char * s8 = (char*) "cdefghijkl";
  char * s9 = (char*) "mnopqrstuv";
  //char * s10 = (char*)"wxyzabcdef";
  //char * s11 =  (char*)"ghijklmnop";
  //char * s12 = (char*) "qrstuvwxyz";
  char * s13 = (char*) "abcdefghij";
  assert(!strncmp( s + 0, s0, 10 ));
  assert(!strncmp( s + 10, s1, 10 ));
  assert(!strncmp( s + 20, s2, 10 ));
  assert(!strncmp( s + 30, s3, 10 ));
  assert(!strncmp( s + 40, s4, 10 ));
  delete data;

  // First copy
  tab->copyRows(1,0,3);

  data = tab -> column( "bool" ) -> data();
  b8 = data->boolData();
  assert(b8[0] == true);
  assert(b8[1] == false);
  assert(b8[2] == true);
  assert(b8[3] == true);
  assert(b8[4] == false);
  delete data;

  data = tab -> column( "int8" ) -> data();
  i8 = data->int8Data();
  assert(i8[0] == 1);
  assert(i8[1] == 2);
  assert(i8[2] == 3);
  assert(i8[3] == 3);
  assert(i8[4] == 4);
  delete data;

  data = tab -> column( "int16" ) -> data();
  i16 = data->int16Data();
  assert(i16[0] == 1);
  assert(i16[1] == 2);
  assert(i16[2] == 3);
  assert(i16[3] == 3);
  assert(i16[4] == 4);
  delete data;

  data = tab -> column( "int32" ) -> data();
  i32 = data->int32Data();
  assert(i32[0] == 1);
  assert(i32[1] == 2);
  assert(i32[2] == 3);
  assert(i32[3] == 3);
  assert(i32[4] == 4);
  delete data;

  data = tab -> column( "real32" ) -> data();
  r32 = data->real32Data();
  assert(r32[0] == 1);
  assert(r32[1] == 2);
  assert(r32[2] == 3);
  assert(r32[3] == 3);
  assert(r32[4] == 4);
  delete data;

  data = tab -> column( "real64" ) -> data();
  r64 = data->real64Data();
  assert(r64[0] == 1);
  assert(r64[1] == 2);
  assert(r64[2] == 3);
  assert(r64[3] == 3);
  assert(r64[4] == 4);
  delete data;

  data = tab -> column( "string" ) -> data();
  s = data->stringData();
  assert(!strncmp( s + 0, s1, 10 ));
  assert(!strncmp( s + 10, s2, 10 ));
  assert(!strncmp( s + 20, s3, 10 ));
  assert(!strncmp( s + 30, s3, 10 ));
  assert(!strncmp( s + 40, s4, 10 ));
  delete data;

  // Second copy
  tab->copyRows(7,3,3);

  data = tab -> column( "bool" ) -> data();
  b8 = data->boolData();
  assert(b8[2] == true);
  assert(b8[3] == true);
  assert(b8[4] == false);
  assert(b8[5] == true);
  assert(b8[6] == false);
  delete data;

  data = tab -> column( "int8" ) -> data();
  i8 = data->int8Data();
  assert(i8[2] == 3);
  assert(i8[3] == 7);
  assert(i8[4] == 8);
  assert(i8[5] == 9);
  assert(i8[6] == 6);
  delete data;

  data = tab -> column( "int16" ) -> data();
  i16 = data->int16Data();
  assert(i16[2] == 3);
  assert(i16[3] == 7);
  assert(i16[4] == 8);
  assert(i16[5] == 9);
  assert(i16[6] == 6);
  delete data;

  data = tab -> column( "int32" ) -> data();
  i32 = data->int32Data();
  assert(i32[2] == 3);
  assert(i32[3] == 7);
  assert(i32[4] == 8);
  assert(i32[5] == 9);
  assert(i32[6] == 6);
  delete data;

  data = tab -> column( "real32" ) -> data();
  r32 = data->real32Data();
  assert(r32[2] == 3);
  assert(r32[3] == 7);
  assert(r32[4] == 8);
  assert(r32[5] == 9);
  assert(r32[6] == 6);
  delete data;

  data = tab -> column( "real64" ) -> data();
  r64 = data->real64Data();
  assert(r64[2] == 3);
  assert(r64[3] == 7);
  assert(r64[4] == 8);
  assert(r64[5] == 9);
  assert(r64[6] == 6);
  delete data;

  data = tab -> column( "string" ) -> data();
  s = data->stringData();
  assert(!strncmp( s + 20, s3, 10 ));
  assert(!strncmp( s + 30, s7, 10 ));
  assert(!strncmp( s + 40, s8, 10 ));
  assert(!strncmp( s + 50, s9, 10 ));
  assert(!strncmp( s + 60, s6, 10 ));
  delete data;

  // Third copy
  tab->copyRows(13,6,1);

  data = tab -> column( "bool" ) -> data();
  b8 = data->boolData();
  assert(b8[5] == true);
  assert(b8[6] == true);
  assert(b8[7] == true);
  delete data;

  data = tab -> column( "int8" ) -> data();
  i8 = data->int8Data();
  assert(i8[5] == 9);
  assert(i8[6] == 13);
  assert(i8[7] == 7);
  delete data;

  data = tab -> column( "int16" ) -> data();
  i16 = data->int16Data();
  assert(i16[5] == 9);
  assert(i16[6] == 13);
  assert(i16[7] == 7);
  delete data;

  data = tab -> column( "int32" ) -> data();
  i32 = data->int32Data();
  assert(i32[5] == 9);
  assert(i32[6] == 13);
  assert(i32[7] == 7);
  delete data;

  data = tab -> column( "real32" ) -> data();
  r32 = data->real32Data();
  assert(r32[5] == 9);
  assert(r32[6] == 13);
  assert(r32[7] == 7);
  delete data;

  data = tab -> column( "real64" ) -> data();
  r64 = data->real64Data();
  assert(r64[5] == 9);
  assert(r64[6] == 13);
  assert(r64[7] == 7);
  delete data;

  data = tab -> column( "string" ) -> data();
  s = data->stringData();
  assert(!strncmp( s + 50, s9, 10 ));
  assert(!strncmp( s + 60, s13, 10 ));
  assert(!strncmp( s + 70, s7, 10 ));
  delete data;

  dataSetServer->close(set);

  return 0;
}
