// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include <Dal.h>
#include <string>
#include <iostream>
#include <cassert>

int main() {

    std::string name("test.dat");
    longVec dim;
    dim.push_back(10);
    DataSet * set = dataSetServer->open(name, Dal::Create);
    Table * tab = set->addTable("Table", 100);
    Column * col = tab->addColumn("Column", Dal::Int32, "", "", dim);
    for(unsigned irow = 0; irow < tab->rows(); ++irow){
	ColumnData * cd = col->rowData(irow);
	int32 * v = cd->int32Data();
	for(unsigned icell = 0; icell < cd->elements(); ++icell){
	    v[icell] = irow*1000 + icell;
	}
	delete cd;
    }

    col = tab->addColumn("ColumnV", Dal::Int32, "", "", Dal::zero);
    for(unsigned irow = 0; irow < tab->rows(); ++irow){
	CellData * cd = col->cellData(irow);
	cd->size(irow);
	int32 * v = cd->int32Data();
	for(unsigned icell = 0; icell < cd->elements(); ++icell){
	    v[icell] = irow*1000 + icell;
	}
	delete cd;
    }

    dataSetServer->close(set);

    set = dataSetServer->open(name, Dal::Read);
    tab = set->table("Table");
    col = tab->column("Column");
    for(unsigned irow = 0; irow < tab->rows(); ++irow){
	ColumnData * cd = col->rowData(irow);
	int32 * v = cd->int32Data();
	for(unsigned icell = 0; icell < cd->elements(); ++icell){
	    long expect = irow*1000 + icell;
	    assert(expect == v[icell]);
	}
	delete cd;
    }

    dataSetServer->close(set);


}
	

