// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
#include<cassert>
#include <Dal.h>

int main()
{
  unsigned long nRows = 1000;

  DataSet* set = dataSetServer->open("test.dat",DataSet::Create);
  Table* tab = set->addTable("someTable",nRows);
  Column *col;
  ColumnData *d;

  unsigned long vectorLen = 5;
  vector<unsigned long> size;
  size.push_back(vectorLen);

  col = tab->addColumn("real32",Column::Real32,"m","real32 data",size);
  assert(col->size() == size);
  assert(col->elements() == vectorLen);

  d = col->data();

  assert(d->elements() == vectorLen);
  assert(d->size() == size);
  assert(d->count() == nRows);

  real32* r32 = d->real32Data();
  for (unsigned long row = d->from(); row < d->from()+d->count(); ++row) {
    for (unsigned long element = 0; element < d->elements(); ++element) {
      unsigned long pos = d->position(row,element);
      assert( pos == row * vectorLen + element );
      r32[d->position(row,element)] = row + 0.1*element;
    }
  }
  delete d;

  size.push_back(2);
  col = tab->addColumn("int32",Column::Int32,"mm","int32 data",size);
  d = col->data();
  int32* i32 = d->int32Data();
  vector<unsigned long> pos(2);
  for (unsigned long row = d->from(); row < d->from()+d->count(); ++row) {
    for (pos[0] = 0; pos[0] < size[0]; ++pos[0])
      for (pos[1] = 0; pos[1] < size[1]; ++pos[1]) {
        unsigned long idx = d->position(row,pos);
        assert( idx ==  row * 10 + pos[0] * 2 + pos[1] );
        i32[idx] = 100*row + 10*pos[0] + pos[1];
      }
  }
  delete d;

  vector<unsigned long> vectorSize;
  vectorSize.push_back( 1000 );
  col = tab->addColumn("real64",Column::Real64,"mm","real64 data",vectorSize);

  for( unsigned long i = 0; i < tab -> rows(); ++i )
  {
    d = col->data( i, 1 );
    real64* r64 = d->real64Data();
    for( unsigned long j = 0; j < col -> elements(); ++ j )
    {
      r64[ j ] = i * col -> elements() + j;
    } // for
    delete d;
  } // for

  dataSetServer->close(set);

  return 0;
}

