// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;


#include <ExtDal.h>
#include <sstream>


class Report : public SubTableProcessor
{

	public:

		Report( Table * table ) : SubTableProcessor( table ) { PROC ENDPROC }

		void proc(SubTable * t) {
			PROC
			clog << "SubTable: " << t -> from() << "," << t -> count() << endl;
			bool8* b =  t -> data( 0 )->boolData();
			int8 * i =  t -> data( 1 )->int8Data();
			clog << bool(*b) << ' ' << int(*i) << endl;

			ENDPROC
    		}

};

	
int main()
{
	unsigned long nRows = 1000;

	DataSet* set = dataSetServer->open("test.dat",DataSet::Create);
	Table* tab = set->addTable("some table",nRows);

	Column * col = tab->addColumn("bool",Column::Bool,"bool data");
	ColumnData * bData = col->data();
	bool8 * b = bData -> boolData();

	col = tab->addColumn("int8",Column::Int8,"int8 data","mm");
	ColumnData * iData = col->data();
	int8 * i8 = iData -> int8Data();

	for (unsigned long i = 0; i < nRows; ++i)
	{
		b[i] = (i % 2 ? true : false );
		i8[i] = i % 100;
	} // for

	delete iData;
	delete bData;

	dataSetServer->close(set);


	set = dataSetServer->open("test.dat",DataSet::Read);

	tab = set->table("some table");
	Report report( tab );
	report.forEachRow();

	dataSetServer->close(set);

	return 0;
}

