// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;


#include<cassert>

#include <ExtDal.h>
#include <sstream>
#include <Tracer.h>
#include <StopWatch.h>


int main()
{
	// *********************************
	// Get clock for timing information.
	// *********************************
	StopWatch		cpuWatch( StopWatch::CPU );
	StopWatch		calenderWatch( StopWatch::Calender );

	unsigned long nRows = 1000;

	DataSet* set = dataSetServer->open("test.dat",DataSet::Create);
	Table* tab = set->addTable("some table",nRows);

	tab->addColumn("bool",Column::Bool,"bool data");
	Column* col = tab->addColumn("int8",Column::Int8,"int8 data","mm");

	Data *d = col->data();
	int8 *i8 = d->int8Data();

	for (unsigned long i = 0; i < nRows; ++i) i8[i] = i % 100;
	delete d;

	dataSetServer->close(set);

	set = dataSetServer->open("test.dat",DataSet::Read );
	tab = set->table("some table");
	{
		SubTable * st = SubTableProcessor( tab ).subTable();
		st -> seek(20,10);
		ColumnData * bData = st -> data( "bool" );
		ColumnData * iData = st -> data( "int8" );

		//st -> seek(20,10);

		bool8* b = bData->boolData();
		b[0] = true;
		b[5] = false;

		int8*  i = iData->int8Data();
		assert( i[0] == 20 );
		assert( i[5] == 25 );

		delete bData;
		delete iData;

		st -> seek(30,10);
		bData = st -> data( "bool" );
		iData = st -> data( "int8" );

		b = bData->boolData();
		b[1] = true;
		b[6] = false;

		i = iData->int8Data();
		assert( i[0] == 30 );
		assert( i[1] == 31 );

		delete bData;
		delete iData;
	}

	dataSetServer->close(set);

	cout << "CPU time: " << cpuWatch.elapsed() / 1000000.0 << " secs" << endl;
	cout << "Calender time: " << calenderWatch.elapsed() << " secs" << endl;

  	return 0;

}

