! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program test

  use dal
  use errorHandling

  type(DataSetT) :: set
  type(ArrayT) :: arr
  type(TableT) :: tab
  type(ColumnT) :: col
  type(AttributeT) :: att

  set = dataSet("test.dat",read)

  att = attribute( set, "sbool" )
  if( dataType( att ) /= BOOLEAN_ATTRIBUTE &
      .or.dataType(set,"sbool") /= BOOLEAN_ATTRIBUTE ) then
    write(*,*) dataType( att )
    call error('internalError',"problem in set bool attribute: wrong data type")
  end if
  write(*,*) booleanAttribute(set,"sbool"), label(set,"sbool")
  if (booleanAttribute(set,"sbool") &
    .or. label(set,"sbool") /= "set bool comment") &
	call error('internalError',"problem in set bool attribute")

  att = attribute( set, "sint8" )
  if( dataType( att ) /= INTEGER_ATTRIBUTE &
      .or.dataType(set,"sint8") /= INTEGER_ATTRIBUTE ) then
    write(*,*) dataType( att )
    call error('internalError',"problem in set int8 attribute: wrong data type")
  end if
  write(*,*) int8Attribute(set,"sint8"),units(set,"sint8"), &
	label(set,"sint8")
  if (int8Attribute(set,"sint8") /= 1_int8 &
    .or. units(set,"sint8") /= "int8 unit" &
    .or. label(set,"sint8") /= "set int8 comment") &
	call error('internalError',"problem in set int8 attribute")

  att = attribute( set, "sint16" )
  if( dataType( att ) /= INTEGER_ATTRIBUTE &
      .or.dataType(set,"sint16") /= INTEGER_ATTRIBUTE ) then
    write(*,*) dataType( att )
    call error('internalError',"problem in set int16 attribute: wrong data type")
  end if
  write(*,*) int16Attribute(set,"sint16"),units(set,"sint16"), &
	label(set,"sint16")
  if (int16Attribute(set,"sint16") /= 2_int16 &
    .or. units(set,"sint16") /= "int16 unit" &
    .or. label(set,"sint16") /= "set int16 comment") &
	call error('internalError',"problem in set int16 attribute")

  att = attribute( set, "sint32" )
  if( dataType( att ) /= INTEGER_ATTRIBUTE &
      .or.dataType(set,"sint32") /= INTEGER_ATTRIBUTE ) then
    write(*,*) dataType( att )
    call error('internalError',"problem in set int32 attribute: wrong data type")
  end if
  write(*,*) int32Attribute(set,"sint32"),units(set,"sint32"), &
	label(set,"sint32")
  if (int32Attribute(set,"sint32") /= 3_int32 &
    .or. units(set,"sint32") /= "int32 unit" &
    .or. label(set,"sint32") /= "set int32 comment") &
	call error('internalError',"problem in set int32 attribute")

  att = attribute( set, "sreal32" )
  if( dataType( att ) /= REAL_ATTRIBUTE &
      .or.dataType(set,"sreal32") /= REAL_ATTRIBUTE ) then
    write(*,*) dataType( att )
    call error('internalError',"problem in set real32 attribute: wrong data type")
  end if
  write(*,*) real32Attribute(set,"sreal32"),units(set,"sreal32"), &
	label(set,"sreal32")
  if (real32Attribute(set,"sreal32") /= 1.5_single &
    .or. units(set,"sreal32") /= "real32 unit" &
    .or. label(set,"sreal32") /= "set real32 comment") &
	call error('internalError',"problem in set real32 attribute")

  att = attribute( set, "sreal64" )
  if( dataType( att ) /= REAL_ATTRIBUTE &
      .or.dataType(set,"sreal64") /= REAL_ATTRIBUTE ) then
    write(*,*) dataType( att )
    call error('internalError',"problem in set real64 attribute: wrong data type")
  end if
  write(*,*) real64Attribute(set,"sreal64"),units(set,"sreal64"), &
	label(set,"sreal64")
  if (real64Attribute(set,"sreal64") /= 2.5_double &
    .or. units(set,"sreal64") /= "real64 unit" &
    .or. label(set,"sreal64") /= "set real64 comment") &
	call error('internalError',"problem in set real64 attribute")

  att = attribute( set, "sstring" )
  if( dataType( att ) /= STRING_ATTRIBUTE &
      .or.dataType(set,"sstring") /= STRING_ATTRIBUTE ) then
    write(*,*) dataType( att )
    ! call error('internalError',"problem in set string attribute: wrong data type")
  end if
  write(*,*) stringAttribute(set,"sstring"),units(set,"sstring"), &
	label(set,"sstring")
  if (stringAttribute(set,"sstring") /= "a set string" &
    .or. label(set,"sstring") /= "set string comment") &
	call error('internalError',"problem in set string attribute")

  arr = array(set,"array",read)
  att = attribute( arr, "abool" )
  if( dataType( att ) /= BOOLEAN_ATTRIBUTE &
      .or.dataType(arr,"abool") /= BOOLEAN_ATTRIBUTE ) then
    write(*,*) dataType( att )
    call error('internalError',"problem in arr bool attribute: wrong data type")
  end if
  write(*,*) booleanAttribute(arr,"abool"), label(arr,"abool")
  if (.not. booleanAttribute(arr,"abool") &
    .or. label(arr,"abool") /= "arr bool comment") &
	call error('internalError',"problem in array bool attribute")

  write(*,*) int8Attribute(arr,"aint8"),units(arr,"aint8"), &
	label(arr,"aint8")
  if (int8Attribute(arr,"aint8") /= 4_int8 &
    .or. units(arr,"aint8") /= "int8 unit" &
    .or. label(arr,"aint8") /= "arr int8 comment") &
	call error('internalError',"problem in array int8 attribute")

  write(*,*) int16Attribute(arr,"aint16"),units(arr,"aint16"), &
	label(arr,"aint16")
  if (int16Attribute(arr,"aint16") /= 5_int16 &
    .or. units(arr,"aint16") /= "int16 unit" &
    .or. label(arr,"aint16") /= "arr int16 comment") &
	call error('internalError',"problem in array int16 attribute")

  write(*,*) int32Attribute(arr,"aint32"),units(arr,"aint32"), &
	label(arr,"aint32")
  if (int32Attribute(arr,"aint32") /= 6_int32 &
    .or. units(arr,"aint32") /= "int32 unit" &
    .or. label(arr,"aint32") /= "arr int32 comment") &
	call error('internalError',"problem in array int32 attribute")

  write(*,*) real32Attribute(arr,"areal32"),units(arr,"areal32"), &
	label(arr,"areal32")
  if (real32Attribute(arr,"areal32") /= 3.5_single &
    .or. units(arr,"areal32") /= "real32 unit" &
    .or. label(arr,"areal32") /= "arr real32 comment") &
	call error('internalError',"problem in array real32 attribute")

  write(*,*) real64Attribute(arr,"areal64"),units(arr,"areal64"), &
	label(arr,"areal64")
  if (real64Attribute(arr,"areal64") /= 4.5_double &
    .or. units(arr,"areal64") /= "real64 unit" &
    .or. label(arr,"areal64") /= "arr real64 comment") &
	call error('internalError',"problem in array real64 attribute")

  write(*,*) stringAttribute(arr,"astring"), label(arr,"astring")
  if (stringAttribute(arr,"astring") /= "an array string" &
    .or. label(arr,"astring") /= "arr string comment") &
	call error('internalError',"problem in array string attribute")

  call release(arr)

  tab = table(set,"table")

  if (booleanAttribute(tab,"abool") &
    .or. label(tab,"abool") /= "tab bool comment") &
	call error('internalError',"problem in table bool attribute")

  if (int8Attribute(tab,"aint8") /= 7_int8 &
    .or. units(tab,"aint8") /= "int8 unit" &
    .or. label(tab,"aint8") /= "tab int8 comment") &
	call error('internalError',"problem in table int8 attribute")

  write(*,*) int16Attribute(tab,"aint16"),units(tab,"aint16"), &
	label(tab,"aint16")
  if (int16Attribute(tab,"aint16") /= 8_int16 &
    .or. units(tab,"aint16") /= "int16 unit" &
    .or. label(tab,"aint16") /= "tab int16 comment") &
	call error('internalError',"problem in table int16 attribute")

  write(*,*) int32Attribute(tab,"aint32"),units(tab,"aint32"), &
	label(tab,"aint32")
  if (int32Attribute(tab,"aint32") /= 9_int32 &
    .or. units(tab,"aint32") /= "int32 unit" &
    .or. label(tab,"aint32") /= "tab int32 comment") &
	call error('internalError',"problem in table int32 attribute")

  write(*,*) real32Attribute(tab,"areal32"),units(tab,"areal32"), &
	label(tab,"areal32")
  if (real32Attribute(tab,"areal32") /= 5.5_single &
    .or. units(tab,"areal32") /= "real32 unit" &
    .or. label(tab,"areal32") /= "tab real32 comment") &
	call error('internalError',"problem in table real32 attribute")

  write(*,*) real64Attribute(tab,"areal64"),units(tab,"areal64"), &
	label(tab,"areal64")
  if (real64Attribute(tab,"areal64") /= 6.5_double &
    .or. units(tab,"areal64") /= "real64 unit" &
    .or. label(tab,"areal64") /= "tab real64 comment") &
	call error('internalError',"problem in table real64 attribute")

  write(*,*) stringAttribute(tab,"astring"),units(tab,"astring"), &
	label(tab,"astring")
  if (stringAttribute(tab,"astring") /= "a table string" &
    .or. label(tab,"astring") /= "tab string comment") &
	call error('internalError',"problem in table string attribute")

  ! ************************
  ! Check column attributes.
  ! ************************
  col = column( tab, "col1", READ )

  write(*,*) int32Attribute(col,"tlmin"),units(col,"tlmin"), &
	label(col,"tlmin")
  if (int32Attribute(col,"tlmin") /= 1_int32 &
    .or. units(col,"tlmin") /= "int32 unit" &
    .or. label(col,"tlmin") /= "col int32 comment") &
	call error('internalError',"problem in column 1 int32 attribute")

  write(*,*) int32Attribute(col,"tlmax"),units(col,"tlmax"), &
	label(col,"tlmax")
  if (int32Attribute(col,"tlmax") /= 2_int32 &
    .or. units(col,"tlmax") /= "int32 unit" &
    .or. label(col,"tlmax") /= "col int32 comment") &
	call error('internalError',"problem in column 1 int32 attribute")

  col = column( tab, "col2", READ )

  write(*,*) int32Attribute(col,"tlmin"),units(col,"tlmin"), &
	label(col,"tlmin")
  if (int32Attribute(col,"tlmin") /= 11_int32 &
    .or. units(col,"tlmin") /= "int32 unit" &
    .or. label(col,"tlmin") /= "col int32 comment") &
	call error('internalError',"problem in column 2 int32 attribute")

  write(*,*) int32Attribute(col,"tlmax"),units(col,"tlmax"), &
	label(col,"tlmax")
  if (int32Attribute(col,"tlmax") /= 22_int32 &
    .or. units(col,"tlmax") /= "int32 unit" &
    .or. label(col,"tlmax") /= "col int32 comment") &
	call error('internalError',"problem in column 2 int32 attribute")


  call release(tab)

  call release(set)

end program test
