! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program test

  use dal
  use errorhandling
  implicit none

  type(DataSetT) set1, set2
  type(ArrayT) array1, array2, array3, array4, array5
  type(TableT) table1, table2, table3, table4, table5
  type(ColumnT) col1, col2, col3

  integer, parameter :: nRows = 10
  integer(kind=int32), dimension(:), pointer :: i32_1, i32_2, i32_3
  integer(kind=int32), dimension(:,:,:), pointer :: a
  integer, dimension(3), parameter :: s = (/ 3,4,2 /)
  integer :: i, j, k, n

  ! -------- CREATE A SET ----------
  set1 = dataSet("test.dat",CREATE)
  call setAttribute(set1,"sbool",.false.,"set1 bool comment")
  call setAttribute(set1,"sint",1,"int unit","set1 int comment")
  call setAttribute(set1,"sreal",2.5,"real unit","set1 real comment")
  call setAttribute(set1,"sstring","a set1 string","set1 string comment")
  ! -------- ADD PRIMARY ARRAY -------
  array1 = addArray(set1, "PRIMARY", INTEGER32, dimensions=s )
  call setAttribute(array1,"RA","(none)","RA of plate centre")
  call setAttribute(array1,"DEC","(none)","DEC of plate centre")
  ! fill with unique numbers
  a => int32array3Data(array1)
  n = 0
  do k=0,1
    do j=0,3
      do i=0,2 
        a(i,j,k) = n
	n = n + 1
      end do
    end do
  end do
  ! -------- ADD A TABLE -----------
  table1 = addTable(set1,"table1",10)
  call setAttribute(table1,attribute(set1,"sbool"));
  ! -------- ADD A COLUMN ----------
  col1 = addColumn(table1,"col1",Integer32,units="dm",comment="int32 column")
  col2 = addColumn(table1,"col2",Integer32,units="dm",comment="int32 column")
  col3 = addColumn(table1,"col3",Integer32,units="dm",comment="int32 column")
  i32_1 => int32Data(col1)
  i32_2 => int32Data(col2)
  i32_3 => int32Data(col3)
  do i=0,nRows-1
    i32_1(i) = 1*i
    i32_2(i) = 2*i
    i32_3(i) = 3*i
  end do
  ! -------- ADD AN ARRAY -------
  array2 = addArray(set1, "array2", INTEGER32, dimensions=s )
  call setAttribute(array2,"RA","(none)","RA of plate centre")
  call setAttribute(array2,"DEC","(none)","DEC of plate centre")
  ! fill with unique numbers
  a => int32array3Data(array2)
  n = 0
  do k=0,1
    do j=0,3
      do i=0,2 
        a(i,j,k) = n
	n = n + 1
      end do
    end do
  end do

  ! -------- ADD A TABLE -----------
  table2 = addTable(set1,"table2",nRows);
  call setAttributes(attributable(table2),attributable(set1));
  ! -------- ADD A COLUMN ----------
  col1 = addColumn(table2,"col1",Integer32,units="dm",comment="int32 column")
  col2 = addColumn(table2,"col2",Integer32,units="dm",comment="int32 column")
  col3 = addColumn(table2,"col3",Integer32,units="dm",comment="int32 column")
  i32_1 => int32Data(col1)
  i32_2 => int32Data(col2)
  i32_3 => int32Data(col3)
  do i=0,nRows-1
    i32_1(i) = 1*i
    i32_2(i) = 2*i
    i32_3(i) = 3*i
  end do
  call release(col1)
  call release(col2)
  call release(col3)

  call copyBlock(set1,block(set1,"array2",READ),"array3");
  call copyBlock(set1,block(array1),"array4");
  call copyBlock(set1,block(set1,"table2",READ),"table3");
  call copyBlock(set1,block(table1),"table4");

  call release(set1)

  call copyDataSet("test.dat","test1.dat")

  ! --------------------------------
  ! check whether copyDataSet worked
  ! --------------------------------

  set2 = dataSet("test1.dat",READ)

  ! ------- CHECK PRIMARY ARRAY --------
  array1 = array(set2,"PRIMARY",READ)
  a => int32array3Data(array1)
  ! test whether the shape of the returned array is identical to the shape
  ! originally specified
  if (any(shape(a) /= s)) then
    write(errstr,*) "shape of returned array",shape(a),"/= array spec",s
    call error("internalError")
  endif

  if (any(dimensions(array1) /= s)) then
    write(errstr,*) "dimensions of array",dimensions(array1),"/= array spec",s
    call error("internalError")
  endif

  n = 0
  do k=0,1
    do j=0,3
      do i=0,2 
        if( a(i,j,k) /= n ) call error("test","PRIMARY value")
	n = n + 1
      end do
    end do
  end do

  if ( stringAttribute(array1,"RA") /= "(none)" ) call error("test","RA")
  if ( stringAttribute(array1,"DEC") /= "(none)" ) call error("test","DEC")


  ! ------- CHECK array2 ARRAY --------
  array2 = array(set2,"array2",READ)
  a => int32array3Data(array2)
  ! test whether the shape of the returned array is identical to the shape
  ! originally specified
  if (any(shape(a) /= s)) then
    write(errstr,*) "shape of returned array",shape(a),"/= array spec",s
    call error("internalError")
  endif

  if (any(dimensions(array2) /= s)) then
    write(errstr,*) "dimensions of array",dimensions(array2),"/= array spec",s
    call error("internalError")
  endif

  n = 0
  do k=0,1
    do j=0,3
      do i=0,2 
        if( a(i,j,k) /= n ) call error("test","PRIMARY value")
	n = n + 1
      end do
    end do
  end do

  if ( stringAttribute(array1,"RA") /= "(none)" ) call error("test","RA")
  if ( stringAttribute(array1,"DEC") /= "(none)" ) call error("test","DEC")

  ! ------- CHECK array3 ARRAY --------
  array3 = array(set2,"array3",READ)
  a => int32array3Data(array3)
  ! test whether the shape of the returned array is identical to the shape
  ! originally specified
  if (any(shape(a) /= s)) then
    write(errstr,*) "shape of returned array",shape(a),"/= array spec",s
    call error("internalError")
  endif

  if (any(dimensions(array3) /= s)) then
    write(errstr,*) "dimensions of array",dimensions(array3),"/= array spec",s
    call error("internalError")
  endif

  n = 0
  do k=0,1
    do j=0,3
      do i=0,2 
        if( a(i,j,k) /= n ) call error("test","array3 value")
	n = n + 1
      end do
    end do
  end do

  if ( stringAttribute(array3,"RA") /= "(none)" ) call error("test","RA")
  if ( stringAttribute(array3,"DEC") /= "(none)" ) call error("test","DEC")

  ! ------- CHECK array4 ARRAY --------
  array4 = array(set2,"array4",READ)
  a => int32array3Data(array4)
  ! test whether the shape of the returned array is identical to the shape
  ! originally specified
  if (any(shape(a) /= s)) then
    write(errstr,*) "shape of returned array",shape(a),"/= array spec",s
    call error("internalError")
  endif

  if (any(dimensions(array4) /= s)) then
    write(errstr,*) "dimensions of array",dimensions(array4),"/= array spec",s
    call error("internalError")
  endif

  n = 0
  do k=0,1
    do j=0,3
      do i=0,2 
        if( a(i,j,k) /= n ) call error("test","array4 value")
	n = n + 1
      end do
    end do
  end do

  if ( stringAttribute(array4,"RA") /= "(none)" ) call error("test","RA")
  if ( stringAttribute(array4,"DEC") /= "(none)" ) call error("test","DEC")

  ! ------- CHECK table3 table --------
  table3 = table(set2,"table3")
  if ( numberOfRows(table3) /= nRows ) call error("test","rows")
  if ( booleanAttribute(table3,"sbool") ) call error("test","sbool")
  if ( int32Attribute(table3,"sint") /= 1) call error("test","sint")
  if ( real32Attribute(table3,"sreal") /= 2.5) call error("test","sreal")
  if ( stringAttribute(table3,"sstring") /= "a set1 string" ) &
	call error("test","sstring")

  col1 = column(table3,"col1",READ)
  col2 = column(table3,"col2",READ)
  col3 = column(table3,"col3",READ)
  if ( units(col1) /= "dm" ) call error("test","units")
  if ( label(col1) /= "int32 column" ) call error("test","label")
  if ( units(col2) /= "dm" ) call error("test","units")
  if ( label(col2) /= "int32 column" ) call error("test","label")
  if ( units(col3) /= "dm" ) call error("test","units")
  if ( label(col3) /= "int32 column" ) call error("test","label")
  i32_1 => int32Data(col1)
  i32_2 => int32Data(col2)
  i32_3 => int32Data(col3)
 
  do i=0,nRows-1
    if ( i32_1(i) /= 1*i ) call error("test","i32 value")
    if ( i32_2(i) /= 2*i ) call error("test","i32 value")
    if ( i32_3(i) /= 3*i ) call error("test","i32 value")
  end do

  ! ------- CHECK table4 table --------
  table4 = table(set2,"table4")
  if ( numberOfRows(table4) /= nRows ) call error("test","rows")
  if ( booleanAttribute(table4,"sbool") ) call error("test","sbool")

  col1 = column(table4,"col1",READ)
  col2 = column(table4,"col2",READ)
  col3 = column(table4,"col3",READ)
  if ( units(col1) /= "dm" ) call error("test","units")
  if ( label(col1) /= "int32 column" ) call error("test","label")
  if ( units(col2) /= "dm" ) call error("test","units")
  if ( label(col2) /= "int32 column" ) call error("test","label")
  if ( units(col3) /= "dm" ) call error("test","units")
  if ( label(col3) /= "int32 column" ) call error("test","label")
  i32_1 => int32Data(col1)
  i32_2 => int32Data(col2)
  i32_3 => int32Data(col3)
 
  do i=0,nRows-1
    if ( i32_1(i) /= 1*i ) call error("test","i32 value")
    if ( i32_2(i) /= 2*i ) call error("test","i32 value")
    if ( i32_3(i) /= 3*i ) call error("test","i32 value")
  end do

  ! check whether copyBlock works from another set
  set1 = dataSet("test.dat",MODIFY)
  call copyBlock(set1,block(array3),"array5")

  ! ------- CHECK array5 ARRAY --------
  array5 = array(set1,"array5",READ)
  a => int32array3Data(array5)
  ! test whether the shape of the returned array is identical to the shape
  ! originally specified
  if (any(shape(a) /= s)) then
    write(errstr,*) "shape of returned array",shape(a),"/= array spec",s
    call error("internalError")
  endif

  if (any(dimensions(array5) /= s)) then
    write(errstr,*) "dimensions of array",dimensions(array5),"/= array spec",s
    call error("internalError")
  endif

  n = 0
  do k=0,1
    do j=0,3
      do i=0,2 
        if( a(i,j,k) /= n ) call error("test","array5 value")
	n = n + 1
      end do
    end do
  end do

  if ( stringAttribute(array5,"RA") /= "(none)" ) call error("test","RA")
  if ( stringAttribute(array5,"DEC") /= "(none)" ) call error("test","DEC")

  call copyBlock(set1,block(table3),"table5")

  ! ------- CHECK table5 TABLE --------
  table5 = table(set1,"table5")
  if ( numberOfRows(table5) /= nRows ) call error("test","rows")
  if ( booleanAttribute(table5,"sbool") ) call error("test","sbool")
  if ( int32Attribute(table5,"sint") /= 1) call error("test","sint")
  if ( real32Attribute(table5,"sreal") /= 2.5) call error("test","sreal")
  if ( stringAttribute(table5,"sstring") /= "a set1 string" ) &
	call error("test","sstring")

  col1 = column(table5,"col1",READ)
  col2 = column(table5,"col2",READ)
  col3 = column(table5,"col3",READ)
  if ( units(col1) /= "dm" ) call error("test","units")
  if ( label(col1) /= "int32 column" ) call error("test","label")
  if ( units(col2) /= "dm" ) call error("test","units")
  if ( label(col2) /= "int32 column" ) call error("test","label")
  if ( units(col3) /= "dm" ) call error("test","units")
  if ( label(col3) /= "int32 column" ) call error("test","label")
  i32_1 => int32Data(col1)
  i32_2 => int32Data(col2)
  i32_3 => int32Data(col3)

  do i=0,nRows-1
    if ( i32_1(i) /= 1*i ) call error("test","i32 value")
    if ( i32_2(i) /= 2*i ) call error("test","i32 value")
    if ( i32_3(i) /= 3*i ) call error("test","i32 value")
  end do


  call release(set2)
  call release(set1)

end program

