! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! This subroutine will write the contents of column 'x' to standard output.
module foreachsubtable1

  use dal
  use errorhandling

contains

  subroutine main()

    ! This part of the program will apply reportX to a table. 
    type(DataSetT) :: set
    type(TableT) :: tab
    type(ColumnT) :: xCol, yCol, zCol
    real(single), dimension(:), pointer :: x, y, z

    set = dataSet("test.dat",CREATE)
    tab = addTable(set,"events",10)
    xCol = addColumn(tab,"x",real32,"mm")
    yCol = addColumn(tab,"y",real32,"mm")
    zCol = addColumn(tab,"z",real32,"mm")

    call forEachSubTable(tab,fill)
    call forEachSubTable(tab,check)
  
    call release(tab)
    call release(set)

  end subroutine

  subroutine fill(tab)
    use dal
    use errorhandling

    type(TableT), intent(in) :: tab

    type(ColumnT) :: xCol, yCol, zCol
    real(kind=single), dimension(:), pointer :: x, y, z

    write(*,*) from( tab ), count( tab )
    xCol = column(tab,"x",READ)
    yCol = column(tab,"y",READ)
    zCol = column(tab,"z",READ)
    x => real32Data(xCol)
    y => real32Data(yCol)
    z => real32Data(zCol)
    x = 3
    write(*,*) x
    y = 4
    write(*,*) y
    z = 5
    write(*,*) z

    call release( xCol );
    call release( yCol );
    call release( zCol );

  end subroutine fill

  subroutine check(tab)
    use dal
    use errorhandling

    type(TableT), intent(in) :: tab

    type(ColumnT) :: xCol, yCol, zCol
    real(kind=single), dimension(:), pointer :: x, y, z

    xCol = column(tab,"x",READ)
    yCol = column(tab,"y",READ)
    zCol = column(tab,"z",READ)
    x => real32Data(xCol)
    y => real32Data(yCol)
    z => real32Data(zCol)
    if (x(0) /= 3.0) call error("","wrong value in column")
    write(*,*) x
    if (y(0) /= 4.0) call error("","wrong value in column")
    write(*,*) y
    if (z(0) /= 5.0) call error("","wrong value in column")
    write(*,*) z

    call release( xCol );
    call release( yCol );
    call release( zCol );

  end subroutine check

end module

