! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program test

  use dal
  use errorhandling

  implicit none

  type(DataSetT) set
  type(ArrayT) arr0, arr1, arr2, arr3
  integer(kind=int16), dimension(:,:,:), pointer :: a0
  integer(kind=int8), dimension(:,:,:), pointer :: a1
  integer(kind=int16), dimension(:,:,:), pointer :: a2
  integer(kind=int32), dimension(:,:,:), pointer :: a3
  integer(kind=int16), dimension(:), pointer :: b0
  integer(kind=int8), dimension(:), pointer :: b1
  integer(kind=int16), dimension(:), pointer :: b2
  integer(kind=int32), dimension(:), pointer :: b3
  integer, dimension(3), parameter :: s = (/ 3,4,2 /)
  integer :: i,j,k,n

  ! create a set
  set = dataSet("test.dat",CREATE)
  arr0 = addArray(set, "PRIMARY", INTEGER16, dimensions=s )
  arr1 = addArray(set, "array1", INTEGER8, dimensions=s )
  arr2 = addArray(set, "array2", INTEGER16, dimensions=s )
  arr3 = addArray(set, "array3", INTEGER32, dimensions=s )
  call setAttribute(arr1,"RA","(none)","RA of plate centre")
  call setAttribute(arr2,"RA","(none)","RA of plate centre")
  call setAttribute(arr3,"RA","(none)","RA of plate centre")

  a0 => int16array3Data(arr0)
  a1 => int8array3Data(arr1)
  a2 => int16array3Data(arr2)
  a3 => int32array3Data(arr3)

  ! test whether the shape of the returned array is identical to the shape
  ! originally specified
  if (any(shape(a0) /= s)) then
    write(errstr,*) "shape of returned array",shape(a0),"/= array spec",s
    call error("internalError")
  endif
  if (any(shape(a1) /= s)) then
    write(errstr,*) "shape of returned array",shape(a1),"/= array spec",s
    call error("internalError")
  endif
  if (any(shape(a2) /= s)) then
    write(errstr,*) "shape of returned array",shape(a2),"/= array spec",s
    call error("internalError")
  endif
  if (any(shape(a3) /= s)) then
    write(errstr,*) "shape of returned array",shape(a3),"/= array spec",s
    call error("internalError")
  endif

  if (any(dimensions(arr0) /= s)) then
    write(errstr,*) "dimensions of array",dimensions(arr0),"/= array spec",s
    call error("internalError")
  endif

  if (any(dimensions(arr1) /= s)) then
    write(errstr,*) "dimensions of array",dimensions(arr1),"/= array spec",s
    call error("internalError")
  endif

  if (any(dimensions(arr2) /= s)) then
    write(errstr,*) "dimensions of array",dimensions(arr2),"/= array spec",s
    call error("internalError")
  endif

  if (any(dimensions(arr3) /= s)) then
    write(errstr,*) "dimensions of array",dimensions(arr3),"/= array spec",s
    call error("internalError")
  endif

  ! fill with unique numbers
  n = 0
  do k=0,1
    do j=0,3
      do i=0,2 
        a0(i,j,k) = n
        a1(i,j,k) = n
        a2(i,j,k) = n
        a3(i,j,k) = n
	    n = n + 1
      end do
    end do
  end do

  do k=0,1
    do j=0,3
  write(*,*) (a0(i,j,k), i=0,2)
    end do
  end do
  do k=0,1
    do j=0,3
  write(*,*) (a1(i,j,k), i=0,2)
    end do
  end do
  do k=0,1
    do j=0,3
  write(*,*) (a2(i,j,k), i=0,2)
    end do
  end do
  do k=0,1
    do j=0,3
  write(*,*) (a3(i,j,k), i=0,2)
    end do
  end do

  call release(arr0)
  call release(arr1)
  call release(arr2)
  call release(arr3)

  ! access it again as vector
  arr0 = array(set,"PRIMARY",READ);
  arr1 = array(set,"array1",READ);
  arr2 = array(set,"array2",READ);
  arr3 = array(set,"array3",READ);
  b0 => int16data(arr0)
  b1 => int8data(arr1)
  b2 => int16data(arr2)
  b3 => int32data(arr3)

  write(*,*) (b0(i), i=0,23)
  write(*,*) (b1(i), i=0,23)
  write(*,*) (b2(i), i=0,23)
  write(*,*) (b3(i), i=0,23)

  ! check the shape
  if ( any(shape(b0) /= (/24/)) ) then
    write(errstr,*) "vector size",shape(b0),"unequal to array size",s
    call error("internalError")
  endif
  if ( any(shape(b1) /= (/24/)) ) then
    write(errstr,*) "vector size",shape(b1),"unequal to array size",s
    call error("internalError")
  endif
  if ( any(shape(b2) /= (/24/)) ) then
    write(errstr,*) "vector size",shape(b2),"unequal to array size",s
    call error("internalError")
  endif
  if ( any(shape(b3) /= (/24/)) ) then
    write(errstr,*) "vector size",shape(b3),"unequal to array size",s
    call error("internalError")
  endif
  
  ! check the values
  do i=0,23
    if ( b0(i) /= i ) then
      write(errstr,*) "vector value",b0(n),"unequal to array value",n
      call error("internalError")
    end if
  end do

  do i=0,23
    if ( b1(i) /= i ) then
      write(errstr,*) "vector value",b1(n),"unequal to array value",n
      call error("internalError")
    end if
  end do

  do i=0,23
    if ( b2(i) /= i ) then
      write(errstr,*) "vector value",b2(n),"unequal to array value",n
      call error("internalError")
    end if
  end do

  do i=0,23
    if ( b3(i) /= i ) then
      write(errstr,*) "vector value",b3(n),"unequal to array value",n
      call error("internalError")
    end if
  end do

  call release(arr0)
  call release(arr1)
  call release(arr2)
  call release(arr3)

  call release(set)

end program test
