! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
subroutine check( set, arr, tab, col1, col2 )

  use dal
  use errorHandling

  type(DataSetT), intent(in) :: set
  type(ArrayT), intent(in) :: arr
  type(TableT), intent(in) :: tab
  type(ColumnT), intent(in) :: col1
  type(ColumnT), intent(in) :: col2

  ! ****************************
  ! Check that attributes exist.
  ! ****************************
  write(*,*) "no. set attributes = ", numberOfAttributes( set )

  !if( numberOfAttributes( set ) .ne. 7 ) then
  !  call error("dalInternalError","set attributes")
  !end if
  if( .not. hasAttribute( set, "sbool" )) then
    call error("dalInternalError","sbool")
  end if
  if( .not. hasAttribute( set, "sint8" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( set, "sint16" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( set, "sint32" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( set, "sreal32" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( set, "sreal64" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( set, "sstring" )) then
    call error("dalInternalError")
  end if
  !if( .not. hasAttribute( set, "empty" )) then
  !  call error("dalInternalError")
  !end if

  write(*,*) "set ok"
  
  if( numberOfAttributes( arr ) .ne. 7 ) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( arr, "abool" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( arr, "aint8" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( arr, "aint16" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( arr, "aint32" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( arr, "areal32" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( arr, "areal64" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( arr, "astring" )) then
    call error("dalInternalError")
  end if

  write(*,*) "array ok"
  
  if( numberOfAttributes( tab ) .ne. 7 ) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( tab, "abool" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( tab, "aint8" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( tab, "aint16" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( tab, "aint32" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( tab, "areal32" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( tab, "areal64" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( tab, "astring" )) then
    call error("dalInternalError")
  end if

  write(*,*) "table ok"
  
  if( numberOfAttributes( col1 ) .ne. 2 ) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( col1, "tlmin" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( col1, "tlmax" )) then
    call error("dalInternalError")
  end if

  if( numberOfAttributes( col2 ) .ne. 2 ) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( col2, "tlmin" )) then
    call error("dalInternalError")
  end if
  if( .not. hasAttribute( col2, "tlmax" )) then
    call error("dalInternalError")
  end if

  write(*,*) "column ok"
  

end subroutine check

program test

  use dal
  use errorHandling

  type(DataSetT) :: set
  type(ArrayT) :: arr
  type(TableT) :: tab
  type(ColumnT) :: col1
  type(ColumnT) :: col2

  set = dataSet("test.dat",create)
  call setAttribute(set,"sbool",.false.,"set bool comment")
  call setAttribute(set,"sint8",1_int8,"int8 unit","set int8 comment")
  call setAttribute(set,"sint16",2_int16,"int16 unit","set int16 comment")
  call setAttribute(set,"sint32",3_int32,"int32 unit","set int32 comment")
  call setAttribute(set,"sreal32",1.5_single,"real32 unit","set real32 comment")
  call setAttribute(set,"sreal64",2.5_double,"real64 unit","set real64 comment")
  call setAttribute(set,"sstring","a set string","set string comment")
  ! call setAttribute(set,"empty","","a string comment" )

  arr = addArray(set,"array",integer32, dimensions=(/ 1,2,3 /))
  call setAttribute(arr,"abool",.true.,"arr bool comment")
  call setAttribute(arr,"aint8",4_int8,"int8 unit","arr int8 comment")
  call setAttribute(arr,"aint16",5_int16,"int16 unit","arr int16 comment")
  call setAttribute(arr,"aint32",6_int32,"int32 unit","arr int32 comment")
  call setAttribute(arr,"areal32",3.5_single,"real32 unit","arr real32 comment")
  call setAttribute(arr,"areal64",4.5_double,"real64 unit","arr real64 comment")
  call setAttribute(arr,"astring","an array string","arr string comment")

  call release(arr)

  tab = addTable(set,"table",10)
  call setAttribute(tab,"abool",.false.,"tab bool comment")
  call setAttribute(tab,"aint8",7_int8,"int8 unit","tab int8 comment")
  call setAttribute(tab,"aint16",8_int16,"int16 unit","tab int16 comment")
  call setAttribute(tab,"aint32",9_int32,"int32 unit","tab int32 comment")
  call setAttribute(tab,"areal32",5.5_single,"real32 unit","tab real32 comment")
  call setAttribute(tab,"areal64",6.5_double,"real64 unit","tab real64 comment")
  call setAttribute(tab,"astring","a table string","tab string comment")

  col1 = addColumn(tab,"col1",INT32,units="UNITS",comment="Column")
  call setAttribute(col1,"tlmin",1_int32,"int32 unit","col int32 comment")
  call setAttribute(col1,"tlmax",2_int32,"int32 unit","col int32 comment")

  col2 = addColumn(tab,"col2",INT32,units="UNITS",comment="Column")
  call setAttribute(col2,"tlmin",11_int32,"int32 unit","col int32 comment")
  call setAttribute(col2,"tlmax",22_int32,"int32 unit","col int32 comment")

  call check( set, arr, tab, col1, col2 )
  call release(set)

  set = dataSet("test.dat",read)
  arr = array(set,"array")
  tab = table(set,"table")
  col1 = column(tab,"col1")
  col2 = column(tab,"col2")
  call check( set, arr, tab, col1, col2 )
  call release(set)

end program test
