! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program test

  use dal
  
  implicit none

  type(DataSetT) set
  type(TableT) tab
  type(ColumnT) col
  logical(kind=bool), dimension(:), pointer :: b
  integer(kind=int8), dimension(:), pointer :: i8
  integer(kind=int16), dimension(:), pointer :: i16
  integer(kind=int32), dimension(:), pointer :: i32
  real(kind=single), dimension(:), pointer :: r32
  real(kind=double), dimension(:), pointer :: r64
  character(len=1024) :: s
  integer i

  set = dataSet("test.dat",CREATE)

  tab = addTable(set,"some table",100)

  col = addColumn(tab,"bool",BOOLEAN)
  b => boolData(col)
  do i=0,numberOfRows(tab)-1
    b(i) = ( modulo (i,2) .eq. 0 )
  end do

  col = addColumn(tab,"int8",INTEGER8,units="cm",comment="int8 column")
  i8 => int8Data(col)
  write(*,*) shape(i8)
  do i=0,numberOfRows(tab)-1
    i8(i) = i
  end do

  col = addColumn(tab,"int16",INTEGER16,units="dm",comment="int16 column")
  i16 => int16Data(col)
  do i=0,numberOfRows(tab)-1
    i16(i) = 2*i
  end do

  col = addColumn(tab,"int32",INTEGER32,units="m",comment="in32 column")
  i32 => int32Data(col)
  do i=0,numberOfRows(tab)-1
    i32(i) = 3*i
  end do

  col = addColumn(tab,"real32",REAL32,units="Dm",comment="real32 column")
  r32 => real32Data(col)
  do i=0,numberOfRows(tab)-1
    r32(i) = 0.5*i
  end do

  col = addColumn(tab,"real64",REAL64,units="hm",comment="real64 column")
  r64 => real64Data(col)
  do i=0,numberOfRows(tab)-1
    r64(i) = 0.25*i
  end do

  col = addColumn(tab,"string",STRING,comment="string column",dimensions=(/80/))
  do i=0,numberOfRows(tab)-1
    write(s,*) "string",i
    call setStringCell(col,i,s)
  end do

  tab = addTable(set,"table2",0)
  col = addColumn(tab,"column",String, "", (/ 19 /))

  do i=0,9
    call insertRows( tab, numberOfRows(tab), 1 )
    write(s,*) "string",i
    call setStringCell(col,i,s)
    write(*,*) stringCell( col, i )
  end do

  call release(set)

end program test
