// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
//
// XMM Science Analysis System
//
// Giuseppe Vacanti (cosine science & computing) 
// September 2001
//
#include<Dal.h>
#include<errstr.h>
#include <ExceptionErrorHandler.h>  
#include<iostream>
#include<Tracer.h>
#include<cmath>
#include<cassert>

int main()
{

  //  tracer.trace(true);



  // SPR-2192
  // Column::insert/delete/copyRows() does not work for variable length vector
  // columns in low mem.

  dataSetServer->client("dal_test");

  const size_t nrows = 100;
  std::string set1 = "spr2192.ds";

  DataSet *s1 = dataSetServer->open(set1, Dal::Create);
  Table *t = s1->addTable("TABLE", nrows);  
  
  {
    Column * c = t->addColumn("real32",Dal::Real32,"","",Dal::zero);
    for(unsigned long k = 0; k < nrows; ++k){      
      CellData *d = c->cellData(k);
      d->size(k+1);
      real32 * v = d->real32Data();
      if(d->size() > 0)
	v[0] = d->size();
      for(unsigned j = 1; j < d->size(); ++j)
	v[j] = k;    
      delete d;
    }
  }
  dataSetServer->close(s1);

  s1 = dataSetServer->open(set1, Dal::Modify);
  t = s1->table("TABLE");

  {
    CellData * cd = t->column("real32")->cellData(0);
    real32 * v = cd->real32Data();
    v[0] = 9999;
    delete cd;
  }
  t->insertRows(0, 1);

  dataSetServer->close(s1);

  s1 = dataSetServer->open(set1, Dal::Modify);
  t = s1->table("TABLE");

  // the first row has not been touched -> size == 0
  CellData * cd = t->column("real32")->cellData(0);
  assert(cd->size() == 0);
  delete cd;

  // the second row (was the first row) has the first element == 9999
  cd = t->column("real32")->cellData(1);
  assert(cd->size() == 1);
  {
    real32 * v = cd->real32Data();
    assert(v[0] == 9999);
  }
  delete cd;

  // the last row
  cd = t->column("real32")->cellData(t->rows() - 1);
  assert(cd->size() == nrows); // not the current table->rows()
  real32 * v = cd->real32Data();
  assert(v[0] == nrows);
  for(unsigned j = 1; j < cd->size(); ++j)
    assert(v[j] == nrows - 1);
  delete cd;

  t->deleteRows(0,3);
  assert(t->rows() == 98);

  unsigned long from = 0;
  unsigned long to = 11;
  unsigned long count = 10;

  dataSetServer->close(s1);
  s1 = dataSetServer->open(set1, Dal::Modify);
  t = s1->table("TABLE");
  t->copyRows(from, to, count);

  dataSetServer->close(s1);
  s1 = dataSetServer->open(set1, Dal::Modify);
  t = s1->table("TABLE");

  for(unsigned long k = 0; k < count; ++k){
    CellData * fd = t->column("real32")->cellData(from + k);
    CellData * td = t->column("real32")->cellData(to + k);
    assert(fd->size() == td->size());
    real32 * fv = fd->real32Data();
    real32 * tv = td->real32Data();
    for(unsigned long l = 0; l < td->size(); ++l)
      assert(fv[l] == tv[l]);
    delete td;
    delete fd;
  }

  
  from = 20;
  to = 10;
  t->copyRows(from, to, count);

  dataSetServer->close(s1);

  s1 = dataSetServer->open(set1, Dal::Modify);
  t = s1->table("TABLE");

  for(unsigned long k = 0; k < count; ++k){
    CellData * fd = t->column("real32")->cellData(from + k);
    CellData * td = t->column("real32")->cellData(to + k);
    assert(fd->size() == td->size());
    real32 * fv = fd->real32Data();
    real32 * tv = td->real32Data();
    for(unsigned long l = 0; l < td->size(); ++l)
      assert(fv[l] == tv[l]);
    delete td;
    delete fd;
  }

  
  dataSetServer->close(s1);

}
